@@ -7,14 +7,20 @@ read a netlist.
 """
 from .simulations import basic_simulation
 from .simulations import thermal_simulation
+from .simulations import thermal_external
 from .utils import interp_current
 from .utils import build_inputs_dict
 from .utils import add_events_to_model
+from .utils import save_to_csv
+from .utils import save_to_npy
+from .utils import save_to_npzcomp
 from .netlist_utils import read_netlist
 from .netlist_utils import setup_circuit
 from .netlist_utils import solve_circuit
 from .netlist_utils import solve_circuit_vectorized
 from .netlist_utils import make_lcapy_circuit
+from .netlist_utils import power_loss
+from .netlist_utils import write_netlist
 from .sim_utils import get_initial_stoichiometries
 from .sim_utils import update_init_conc
 from .solver_utils import solve
@@ -33,10 +39,10 @@ from .logger import logger, set_logging_level, log_to_file
 from .definitions import ROOT_DIR
 from .definitions import MODULE_DIR
 from .definitions import CIRCUIT_DIR
-from .solvers import casadi_manager
-from .solvers import ray_manager
-from .solvers import dask_manager
-from .solvers import generic_actor
-from .solvers import ray_actor
+from .solvers import CasadiManager
+from .solvers import RayManager
+from .solvers import DaskManager
+from .solvers import GenericActor
+from .solvers import RayActor
 
-__version__ = "0.3"
+__version__ = "0.3.1"
